@@ -49,6 +49,15 @@ EXPECTED_STATES = {
 }
 """Mapping of provisioning actions to expected stable states."""
 
+EXPECTED_POWER_STATES = {
+    'power on': 'power on',
+    'power off': 'power off',
+    'rebooting': 'power on',
+    'soft power off': 'power off',
+    'soft rebooting': 'power on',
+}
+"""Mapping of target power states to expected power states."""
+
 STATE_VERSIONS = {
     'enroll': '1.11',
     'manageable': '1.4',
@@ -58,8 +67,17 @@ STATE_VERSIONS = {
 VIF_VERSION = '1.28'
 """API version in which the VIF operations were introduced."""
 
+CONFIG_DRIVE_REBUILD_VERSION = '1.35'
+"""API version in which rebuild accepts a configdrive."""
 
-class ListMixin(object):
+RESET_INTERFACES_VERSION = '1.45'
+"""API version in which the reset_interfaces parameter was introduced."""
+
+CONFIG_DRIVE_DICT_VERSION = '1.56'
+"""API version in which configdrive can be a dictionary."""
+
+
+class ListMixin:
 
     @classmethod
     def list(cls, session, details=False, **params):
