@@ -23,12 +23,15 @@ from openstack.compute.v2 import server as _server
 from openstack.compute.v2 import server_diagnostics as _server_diagnostics
 from openstack.compute.v2 import server_group as _server_group
 from openstack.compute.v2 import server_interface as _server_interface
+from openstack.compute.v2 import (
+    server_remote_console as _server_remote_console)
 from openstack.compute.v2 import server_ip
 from openstack.compute.v2 import service as _service
 from openstack.compute.v2 import volume_attachment as _volume_attachment
 from openstack.network.v2 import security_group as _sg
 from openstack import proxy
 from openstack import resource
+from openstack import utils
 
 
 class Proxy(proxy.Proxy):
@@ -56,19 +59,27 @@ class Proxy(proxy.Proxy):
         """
         return self._list(extension.Extension)
 
-    def find_flavor(self, name_or_id, ignore_missing=True):
+    # ========== Flavors ==========
+
+    def find_flavor(self, name_or_id, ignore_missing=True,
+                    get_extra_specs=False):
         """Find a single flavor
 
         :param name_or_id: The name or ID of a flavor.
         :param bool ignore_missing: When set to ``False``
-                    :class:`~openstack.exceptions.ResourceNotFound` will be
-                    raised when the resource does not exist.
-                    When set to ``True``, None will be returned when
-                    attempting to find a nonexistent resource.
+            :class:`~openstack.exceptions.ResourceNotFound` will be raised when
+            the resource does not exist.  When set to ``True``, None will be
+            returned when attempting to find a nonexistent resource.
+        :param bool get_extra_specs: When set to ``True`` and extra_specs not
+            present in the response will invoke additional API call to fetch
+            extra_specs.
         :returns: One :class:`~openstack.compute.v2.flavor.Flavor` or None
         """
-        return self._find(_flavor.Flavor, name_or_id,
-                          ignore_missing=ignore_missing)
+        flavor = self._find(_flavor.Flavor, name_or_id,
+                            ignore_missing=ignore_missing)
+        if flavor and get_extra_specs and not flavor.extra_specs:
+            flavor = flavor.fetch_extra_specs(self)
+        return flavor
 
     def create_flavor(self, **attrs):
         """Create a new flavor from attributes
@@ -97,32 +108,134 @@ class Proxy(proxy.Proxy):
         """
         self._delete(_flavor.Flavor, flavor, ignore_missing=ignore_missing)
 
-    def get_flavor(self, flavor):
+    def get_flavor(self, flavor, get_extra_specs=False):
         """Get a single flavor
 
         :param flavor: The value can be the ID of a flavor or a
-                       :class:`~openstack.compute.v2.flavor.Flavor` instance.
+            :class:`~openstack.compute.v2.flavor.Flavor` instance.
+        :param bool get_extra_specs: When set to ``True`` and extra_specs not
+            present in the response will invoke additional API call to fetch
+            extra_specs.
 
         :returns: One :class:`~openstack.compute.v2.flavor.Flavor`
         :raises: :class:`~openstack.exceptions.ResourceNotFound`
-                 when no resource can be found.
+            when no resource can be found.
         """
-        return self._get(_flavor.Flavor, flavor)
+        flavor = self._get(_flavor.Flavor, flavor)
+        if get_extra_specs and not flavor.extra_specs:
+            flavor = flavor.fetch_extra_specs(self)
+        return flavor
 
     def flavors(self, details=True, **query):
         """Return a generator of flavors
 
         :param bool details: When ``True``, returns
-            :class:`~openstack.compute.v2.flavor.FlavorDetail` objects,
-            otherwise :class:`~openstack.compute.v2.flavor.Flavor`.
-            *Default: ``True``*
+            :class:`~openstack.compute.v2.flavor.Flavor` objects,
+            with additional attributes filled.
         :param kwargs query: Optional query parameters to be sent to limit
-                                 the flavors being returned.
+            the flavors being returned.
 
         :returns: A generator of flavor objects
         """
-        flv = _flavor.FlavorDetail if details else _flavor.Flavor
-        return self._list(flv, **query)
+        base_path = '/flavors/detail' if details else '/flavors'
+        return self._list(_flavor.Flavor, base_path=base_path, **query)
+
+    def flavor_add_tenant_access(self, flavor, tenant):
+        """Adds tenant/project access to flavor.
+
+        :param flavor: Either the ID of a flavor or a
+            :class:`~openstack.compute.v2.flavor.Flavor` instance.
+        :param str tenant: The UUID of the tenant.
+
+        :returns: One :class:`~openstack.compute.v2.flavor.Flavor`
+        """
+        flavor = self._get_resource(_flavor.Flavor, flavor)
+        return flavor.add_tenant_access(self, tenant)
+
+    def flavor_remove_tenant_access(self, flavor, tenant):
+        """Removes tenant/project access to flavor.
+
+        :param flavor: Either the ID of a flavor or a
+            :class:`~openstack.compute.v2.flavor.Flavor` instance.
+        :param str tenant: The UUID of the tenant.
+
+        :returns: One :class:`~openstack.compute.v2.flavor.Flavor`
+        """
+        flavor = self._get_resource(_flavor.Flavor, flavor)
+        return flavor.remove_tenant_access(self, tenant)
+
+    def get_flavor_access(self, flavor):
+        """Lists tenants who have access to private flavor
+
+        :param flavor: Either the ID of a flavor or a
+            :class:`~openstack.compute.v2.flavor.Flavor` instance.
+
+        :returns: List of dicts with flavor_id and tenant_id attributes.
+        """
+        flavor = self._get_resource(_flavor.Flavor, flavor)
+        return flavor.get_access(self)
+
+    def fetch_flavor_extra_specs(self, flavor):
+        """Lists Extra Specs of a flavor
+
+        :param flavor: Either the ID of a flavor or a
+            :class:`~openstack.compute.v2.flavor.Flavor` instance.
+
+        :returns: One :class:`~openstack.compute.v2.flavor.Flavor`
+        """
+        flavor = self._get_resource(_flavor.Flavor, flavor)
+        return flavor.fetch_extra_specs(self)
+
+    def create_flavor_extra_specs(self, flavor, extra_specs):
+        """Lists Extra Specs of a flavor
+
+        :param flavor: Either the ID of a flavor or a
+            :class:`~openstack.compute.v2.flavor.Flavor` instance.
+        :param dict extra_specs: dict of extra specs
+
+        :returns: One :class:`~openstack.compute.v2.flavor.Flavor`
+        """
+        flavor = self._get_resource(_flavor.Flavor, flavor)
+        return flavor.create_extra_specs(self, specs=extra_specs)
+
+    def get_flavor_extra_specs_property(self, flavor, prop):
+        """Get specific Extra Spec property of a flavor
+
+        :param flavor: Either the ID of a flavor or a
+            :class:`~openstack.compute.v2.flavor.Flavor` instance.
+        :param str prop: Property name.
+
+        :returns: String value of the requested property.
+        """
+        flavor = self._get_resource(_flavor.Flavor, flavor)
+        return flavor.get_extra_specs_property(self, prop)
+
+    def update_flavor_extra_specs_property(self, flavor, prop, val):
+        """Update specific Extra Spec property of a flavor
+
+        :param flavor: Either the ID of a flavor or a
+            :class:`~openstack.compute.v2.flavor.Flavor` instance.
+        :param str prop: Property name.
+        :param str val: Property value.
+
+        :returns: String value of the requested property.
+        """
+        flavor = self._get_resource(_flavor.Flavor, flavor)
+        return flavor.update_extra_specs_property(self, prop, val)
+
+    def delete_flavor_extra_specs_property(self, flavor, prop):
+        """Delete specific Extra Spec property of a flavor
+
+        :param flavor: Either the ID of a flavor or a
+            :class:`~openstack.compute.v2.flavor.Flavor` instance.
+        :param str prop: Property name.
+
+        :returns: None
+        """
+        flavor = self._get_resource(_flavor.Flavor, flavor)
+        return flavor.delete_extra_specs_property(self, prop)
+
+    # ========== Aggregates ==========
 
     def aggregates(self, **query):
         """Return a generator of aggregate
@@ -1473,6 +1586,62 @@ class Proxy(proxy.Proxy):
         return self._get(_server_diagnostics.ServerDiagnostics,
                          server_id=server_id, requires_id=False)
 
+    def create_server_remote_console(self, server, **attrs):
+        """Create a remote console on the server.
+
+        :param server: Either the ID of a server or a
+                       :class:`~openstack.compute.v2.server.Server` instance.
+        :returns: One
+            :class:`~openstack.compute.v2.server_remote_console.
+                    ServerRemoteConsole`
+        """
+        server_id = resource.Resource._get_id(server)
+        return self._create(_server_remote_console.ServerRemoteConsole,
+                            server_id=server_id, **attrs)
+
+    def get_server_console_url(self, server, console_type):
+        """Create a remote console on the server.
+
+        :param server: Either the ID of a server or a
+            :class:`~openstack.compute.v2.server.Server` instance.
+        :param console_type: Type of the console connection.
+        :returns: Dictionary with console type and url
+        """
+        server = self._get_resource(_server.Server, server)
+        return server.get_console_url(self, console_type)
+
+    def create_console(self, server, console_type, console_protocol=None):
+        """Create a remote console on the server.
+
+        When microversion supported is higher then 2.6 remote console is
+        created, otherwise deprecated call to get server console is issued.
+
+        :param server: Either the ID of a server or a
+            :class:`~openstack.compute.v2.server.Server` instance.
+        :param console_type: Type of the remote console. Supported values as:
+            * novnc
+            * spice-html5
+            * rdp-html5
+            * serial
+            * webmks (supported after 2.8)
+        :param console_protocol: Optional console protocol (is respected only
+            after microversion 2.6).
+
+        :returns: Dictionary with console type, url and optionally protocol.
+        """
+        server = self._get_resource(_server.Server, server)
+        # NOTE: novaclient supports undocumented type xcpvnc also supported
+        # historically by OSC. We support it, but do not document either.
+        if utils.supports_microversion(self, '2.6'):
+            console = self._create(
+                _server_remote_console.ServerRemoteConsole,
+                server_id=server.id,
+                type=console_type,
+                protocol=console_protocol)
+            return console.to_dict()
+        else:
+            return server.get_console_url(self, console_type)
+
     def _get_cleanup_dependencies(self):
         return {
             'compute': {
@@ -1480,9 +1649,28 @@ class Proxy(proxy.Proxy):
             }
         }
 
-    def _service_cleanup(self, dry_run=True, status_queue=None):
-        for obj in self.servers(details=False):
-            self._service_cleanup_del_res(self.delete_server,
-                                          obj,
-                                          dry_run,
-                                          status_queue)
+    def _service_cleanup(self, dry_run=True, client_status_queue=None,
+                         identified_resources=None,
+                         filters=None, resource_evaluation_fn=None):
+        servers = []
+        for obj in self.servers():
+            need_delete = self._service_cleanup_del_res(
+                self.delete_server,
+                obj,
+                dry_run=dry_run,
+                client_status_queue=client_status_queue,
+                identified_resources=identified_resources,
+                filters=filters,
+                resource_evaluation_fn=resource_evaluation_fn)
+            if not dry_run and need_delete:
+                # In the dry run we identified, that server will go. To propely
+                # identify consequences we need to tell others, that the port
+                # will disappear as well
+                for port in self._connection.network.ports(device_id=obj.id):
+                    identified_resources[port.id] = port
+                servers.append(obj)
+
+        # We actually need to wait for servers to really disappear, since they
+        # might be still holding ports on the subnet
+        for server in servers:
+            self.wait_for_delete(server)
