@@ -34,6 +34,7 @@ and then returned to the caller.
 import collections
 import inspect
 import itertools
+import urllib.parse
 
 import jsonpatch
 import operator
@@ -41,7 +42,6 @@ from keystoneauth1 import adapter
 from keystoneauth1 import discover
 import munch
 from requests import structures
-import six
 
 from openstack import _log
 from openstack import exceptions
@@ -50,6 +50,8 @@ from openstack import utils
 
 _SEEN_FORMAT = '{name}_seen'
 
+LOG = _log.setup_logging(__name__)
+
 
 def _convert_type(value, data_type, list_type=None):
     # This should allow handling list of dicts that have their own
@@ -82,15 +84,25 @@ def _convert_type(value, data_type, list_type=None):
         return data_type(value)
 
 
-class _BaseComponent(object):
+class _BaseComponent:
 
     # The name this component is being tracked as in the Resource
     key = None
     # The class to be used for mappings
     _map_cls = dict
 
+    #: Marks the property as deprecated.
+    deprecated = False
+    #: Deprecation reason message used to warn users when deprecated == True
+    deprecation_reason = None
+
+    #: Control field used to manage the deprecation warning. We want to warn
+    # only once when the attribute is retrieved in the code.
+    already_warned_deprecation = False
+
     def __init__(self, name, type=None, default=None, alias=None, aka=None,
                  alternate_id=False, list_type=None, coerce_to_default=False,
+                 deprecated=False, deprecation_reason=None,
                  **kwargs):
         """A typed descriptor for a component that makes up a Resource
 
@@ -115,6 +127,11 @@ class _BaseComponent(object):
             If the Component is None or not present, force the given default
             to be used. If a default is not given but a type is given,
             construct an empty version of the type in question.
+        :param deprecated:
+            Indicates if the option is deprecated. If it is, we display a
+            warning message to the user.
+        :param deprecation_reason:
+            Custom deprecation message.
         """
         self.name = name
         self.type = type
@@ -128,6 +145,9 @@ class _BaseComponent(object):
         self.list_type = list_type
         self.coerce_to_default = coerce_to_default
 
+        self.deprecated = deprecated
+        self.deprecation_reason = deprecation_reason
+
     def __get__(self, instance, owner):
         if instance is None:
             return self
@@ -137,6 +157,7 @@ class _BaseComponent(object):
         try:
             value = attributes[self.name]
         except KeyError:
+            value = self.default
             if self.alias:
                 # Resource attributes can be aliased to each other. If neither
                 # of them exist, then simply doing a
@@ -156,15 +177,29 @@ class _BaseComponent(object):
                     setattr(instance, seen_flag, True)
                     value = getattr(instance, self.alias)
                     delattr(instance, seen_flag)
-                    return value
-            return self.default
+            self.warn_if_deprecated_property(value)
+            return value
 
         # self.type() should not be called on None objects.
         if value is None:
             return None
 
+        self.warn_if_deprecated_property(value)
         return _convert_type(value, self.type, self.list_type)
 
+    def warn_if_deprecated_property(self, value):
+        deprecated = object.__getattribute__(self, 'deprecated')
+        deprecate_reason = object.__getattribute__(self, 'deprecation_reason')
+
+        if value and deprecated and not self.already_warned_deprecation:
+            self.already_warned_deprecation = True
+            if not deprecate_reason:
+                LOG.warning("The option [%s] has been deprecated. "
+                            "Please avoid using it.", self.name)
+            else:
+                LOG.warning(deprecate_reason)
+        return value
+
     def __set__(self, instance, value):
         if self.coerce_to_default and value is None:
             value = self.default
@@ -207,7 +242,7 @@ class Computed(_BaseComponent):
     key = "_computed"
 
 
-class _ComponentManager(collections.MutableMapping):
+class _ComponentManager(collections.abc.MutableMapping):
     """Storage of a component type"""
 
     def __init__(self, attributes=None, synchronized=False):
@@ -257,7 +292,7 @@ class _ComponentManager(collections.MutableMapping):
             self._dirty = set()
 
 
-class _Request(object):
+class _Request:
     """Prepared components that go into a KSA request"""
 
     def __init__(self, url, body, headers):
@@ -266,7 +301,7 @@ class _Request(object):
         self.headers = headers
 
 
-class QueryParameters(object):
+class QueryParameters:
 
     def __init__(self, *names, **mappings):
         """Create a dict of accepted query parameters
@@ -348,7 +383,7 @@ class QueryParameters(object):
             # single-argument (like int) and double-argument type functions.
             try:
                 provide_resource_type = (
-                    len(inspect.getargspec(type_).args) > 1)
+                    len(inspect.getfullargspec(type_).args) > 1)
             except TypeError:
                 provide_resource_type = False
 
@@ -436,6 +471,8 @@ class Resource(dict):
     requires_id = True
     #: Whether create requires an ID (determined from method if None).
     create_requires_id = None
+    #: Whether create should exclude ID in the body of the request.
+    create_exclude_id_from_body = False
     #: Do responses for this resource have bodies
     has_body = True
     #: Does create returns a body (if False requires ID), defaults to has_body
@@ -560,6 +597,14 @@ class Resource(dict):
             self._computed.attributes == comparand._computed.attributes
         ])
 
+    def warning_if_attribute_deprecated(self, attr, value):
+        if value and self.deprecated:
+            if not self.deprecation_reason:
+                LOG.warning("The option [%s] has been deprecated. "
+                            "Please avoid using it.", attr)
+            else:
+                LOG.warning(self.deprecation_reason)
+
     def __getattribute__(self, name):
         """Return an attribute on this instance
 
@@ -573,7 +618,9 @@ class Resource(dict):
             else:
                 try:
                     return self._body[self._alternate_id()]
-                except KeyError:
+                except KeyError as e:
+                    LOG.debug("Attribute [%s] not found in [%s]: %s.",
+                              self._alternate_id(), self._body, e)
                     return None
         else:
             try:
@@ -1092,7 +1139,7 @@ class Resource(dict):
             uri = utils.urljoin(uri, self.id)
 
         if params:
-            query_params = six.moves.urllib.parse.urlencode(params)
+            query_params = urllib.parse.urlencode(params)
             uri += '?' + query_params
 
         return _Request(uri, body, headers)
@@ -1115,6 +1162,13 @@ class Resource(dict):
                 if self.resource_key and self.resource_key in body:
                     body = body[self.resource_key]
 
+                # Do not allow keys called "self" through. Glance chose
+                # to name a key "self", so we need to pop it out because
+                # we can't send it through cls.existing and into the
+                # Resource initializer. "self" is already the first
+                # argument and is practically a reserved word.
+                body.pop("self", None)
+
                 body_attrs = self._consume_body_attrs(body)
 
                 if self._store_unknown_attrs_as_properties:
@@ -1219,7 +1273,10 @@ class Resource(dict):
             raise exceptions.NotSupported(message)
 
         actual = self._get_microversion_for(session, action)
-        if actual is None:
+
+        if expected is None:
+            return actual
+        elif actual is None:
             message = ("API version %s is required, but the default "
                        "version will be used.") % expected
             _raise(message)
@@ -1258,6 +1315,10 @@ class Resource(dict):
         requires_id = (self.create_requires_id
                        if self.create_requires_id is not None
                        else self.create_method == 'PUT')
+
+        if self.create_exclude_id_from_body:
+            self._body._dirty.discard("id")
+
         if self.create_method == 'PUT':
             request = self._prepare_request(requires_id=requires_id,
                                             prepend_key=prepend_key,
@@ -1767,11 +1828,10 @@ class Resource(dict):
         # This prevents duplication of query parameters that with large
         # number of pages result in HTTP 414 error eventually.
         if next_link:
-            parts = six.moves.urllib.parse.urlparse(next_link)
-            query_params = six.moves.urllib.parse.parse_qs(parts.query)
+            parts = urllib.parse.urlparse(next_link)
+            query_params = urllib.parse.parse_qs(parts.query)
             params.update(query_params)
-            next_link = six.moves.urllib.parse.urljoin(next_link,
-                                                       parts.path)
+            next_link = urllib.parse.urljoin(next_link, parts.path)
 
         # If we still have no link, and limit was given and is non-zero,
         # and the number of records yielded equals the limit, then the user
@@ -1854,7 +1914,7 @@ class Resource(dict):
             "No %s found for %s" % (cls.__name__, name_or_id))
 
 
-class TagMixin(object):
+class TagMixin:
 
     _tag_query_parameters = {
         'tags': 'tags',
@@ -1999,7 +2059,6 @@ def wait_for_status(session, resource, status, failures, interval=None,
     :raises: :class:`~AttributeError` if the resource does not have a status
              attribute
     """
-    log = _log.setup_logging(__name__)
 
     current_status = getattr(resource, attribute)
     if _normalize_status(current_status) == status.lower():
@@ -2033,7 +2092,7 @@ def wait_for_status(session, resource, status, failures, interval=None,
                 "{name} transitioned to failure state {status}".format(
                     name=name, status=new_status))
 
-        log.debug('Still waiting for resource %s to reach state %s, '
+        LOG.debug('Still waiting for resource %s to reach state %s, '
                   'current state is %s', name, status, new_status)
 
 
