@@ -14,7 +14,6 @@ import os
 import openstack.config
 
 from keystoneauth1 import discover
-from keystoneauth1 import exceptions as _exceptions
 from openstack import connection
 from openstack.tests import base
 
@@ -23,15 +22,14 @@ from openstack.tests import base
 #: file, typically in $HOME/.config/openstack/clouds.yaml. That configuration
 #: will determine where the functional tests will be run and what resource
 #: defaults will be used to run the functional tests.
+TEST_CONFIG = openstack.config.OpenStackConfig()
 TEST_CLOUD_NAME = os.getenv('OS_CLOUD', 'devstack-admin')
 TEST_CLOUD_REGION = openstack.config.get_cloud_region(cloud=TEST_CLOUD_NAME)
 
 
 def _get_resource_value(resource_key, default):
-    try:
-        return TEST_CLOUD_REGION.config['functional'][resource_key]
-    except KeyError:
-        return default
+    return TEST_CONFIG.get_extra_config(
+        'functional').get(resource_key, default)
 
 
 def _disable_keep_alive(conn):
@@ -39,20 +37,13 @@ def _disable_keep_alive(conn):
     sess.keep_alive = False
 
 
-IMAGE_NAME = _get_resource_value('image_name', 'cirros-0.3.6-x86_64-disk')
+IMAGE_NAME = _get_resource_value('image_name', 'cirros-0.4.0-x86_64-disk')
 FLAVOR_NAME = _get_resource_value('flavor_name', 'm1.small')
 
 
 class BaseFunctionalTest(base.TestCase):
 
-    @classmethod
-    def setUpClass(cls):
-        super(BaseFunctionalTest, cls).setUpClass()
-        # Defines default timeout for wait_for methods used
-        # in the functional tests
-        cls._wait_for_timeout = int(os.getenv(
-            'OPENSTACKSDK_FUNC_TEST_TIMEOUT',
-            300))
+    _wait_for_timeout_key = ''
 
     def setUp(self):
         super(BaseFunctionalTest, self).setUp()
@@ -60,6 +51,8 @@ class BaseFunctionalTest(base.TestCase):
         _disable_keep_alive(self.conn)
 
         self._demo_name = os.environ.get('OPENSTACKSDK_DEMO_CLOUD', 'devstack')
+        self._demo_name_alt = os.environ.get('OPENSTACKSDK_DEMO_CLOUD_ALT',
+                                             'devstack-alt')
         self._op_name = os.environ.get(
             'OPENSTACKSDK_OPERATOR_CLOUD', 'devstack-admin')
 
@@ -70,12 +63,25 @@ class BaseFunctionalTest(base.TestCase):
         self.identity_version = \
             self.operator_cloud.config.get_api_version('identity')
 
+        # Defines default timeout for wait_for methods used
+        # in the functional tests
+        self._wait_for_timeout = int(
+            os.getenv(self._wait_for_timeout_key, os.getenv(
+                'OPENSTACKSDK_FUNC_TEST_TIMEOUT', 300)))
+
     def _set_user_cloud(self, **kwargs):
         user_config = self.config.get_one(
             cloud=self._demo_name, **kwargs)
         self.user_cloud = connection.Connection(config=user_config)
         _disable_keep_alive(self.user_cloud)
 
+        # This cloud is used by the project_cleanup test, so you can't rely on
+        # it
+        user_config_alt = self.config.get_one(
+            cloud=self._demo_name_alt, **kwargs)
+        self.user_cloud_alt = connection.Connection(config=user_config_alt)
+        _disable_keep_alive(self.user_cloud_alt)
+
     def _set_operator_cloud(self, **kwargs):
         operator_config = self.config.get_one(
             cloud=self._op_name, **kwargs)
@@ -115,8 +121,6 @@ class BaseFunctionalTest(base.TestCase):
             self.assertIsNone(result)
         self.addCleanup(cleanup)
 
-    # TODO(shade) Replace this with call to conn.has_service when we've merged
-    #             the shade methods into Connection.
     def require_service(self, service_type, min_microversion=None, **kwargs):
         """Method to check whether a service exists
 
@@ -129,16 +133,16 @@ class BaseFunctionalTest(base.TestCase):
 
         :returns: True if the service exists, otherwise False.
         """
-        try:
-            data = self.conn.session.get_endpoint_data(
-                service_type=service_type, **kwargs)
-        except _exceptions.EndpointNotFound:
+        if not self.conn.has_service(service_type):
             self.skipTest('Service {service_type} not found in cloud'.format(
                 service_type=service_type))
 
         if not min_microversion:
             return
 
+        data = self.conn.session.get_endpoint_data(
+            service_type=service_type, **kwargs)
+
         if not (data.min_microversion
                 and data.max_microversion
                 and discover.version_between(
