@@ -9,6 +9,7 @@
 # WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 # License for the specific language governing permissions and limitations
 # under the License.
+from unittest import mock
 
 from openstack.compute.v2 import _proxy
 from openstack.compute.v2 import availability_zone as az
@@ -22,6 +23,7 @@ from openstack.compute.v2 import server
 from openstack.compute.v2 import server_group
 from openstack.compute.v2 import server_interface
 from openstack.compute.v2 import server_ip
+from openstack.compute.v2 import server_remote_console
 from openstack.compute.v2 import service
 from openstack.tests.unit import test_proxy_base
 
@@ -31,12 +33,8 @@ class TestComputeProxy(test_proxy_base.TestProxyBase):
         super(TestComputeProxy, self).setUp()
         self.proxy = _proxy.Proxy(self.session)
 
-    def test_extension_find(self):
-        self.verify_find(self.proxy.find_extension, extension.Extension)
-
-    def test_extensions(self):
-        self.verify_list_no_kwargs(self.proxy.extensions, extension.Extension)
 
+class TestFlavor(TestComputeProxy):
     def test_flavor_create(self):
         self.verify_create(self.proxy.create_flavor, flavor.Flavor)
 
@@ -49,18 +47,161 @@ class TestComputeProxy(test_proxy_base.TestProxyBase):
     def test_flavor_find(self):
         self.verify_find(self.proxy.find_flavor, flavor.Flavor)
 
-    def test_flavor_get(self):
-        self.verify_get(self.proxy.get_flavor, flavor.Flavor)
+    def test_flavor_find_fetch_extra(self):
+        """fetch extra_specs is triggered"""
+        with mock.patch(
+            'openstack.compute.v2.flavor.Flavor.fetch_extra_specs'
+        ) as mocked:
+            res = flavor.Flavor()
+            mocked.return_value = res
+            self._verify2(
+                'openstack.proxy.Proxy._find',
+                self.proxy.find_flavor,
+                method_args=['res', True, True],
+                expected_result=res,
+                expected_args=[flavor.Flavor, 'res'],
+                expected_kwargs={'ignore_missing': True}
+            )
+            mocked.assert_called_once()
+
+    def test_flavor_find_skip_fetch_extra(self):
+        """fetch extra_specs not triggered"""
+        with mock.patch(
+            'openstack.compute.v2.flavor.Flavor.fetch_extra_specs'
+        ) as mocked:
+            res = flavor.Flavor(extra_specs={'a': 'b'})
+            mocked.return_value = res
+            self._verify2(
+                'openstack.proxy.Proxy._find',
+                self.proxy.find_flavor,
+                method_args=['res', True],
+                expected_result=res,
+                expected_args=[flavor.Flavor, 'res'],
+                expected_kwargs={'ignore_missing': True}
+            )
+            mocked.assert_not_called()
+
+    def test_flavor_get_no_extra(self):
+        """fetch extra_specs not triggered"""
+        with mock.patch(
+            'openstack.compute.v2.flavor.Flavor.fetch_extra_specs'
+        ) as mocked:
+            res = flavor.Flavor()
+            mocked.return_value = res
+            self._verify2(
+                'openstack.proxy.Proxy._get',
+                self.proxy.get_flavor,
+                method_args=['res'],
+                expected_result=res,
+                expected_args=[flavor.Flavor, 'res']
+            )
+            mocked.assert_not_called()
+
+    def test_flavor_get_fetch_extra(self):
+        """fetch extra_specs is triggered"""
+        with mock.patch(
+            'openstack.compute.v2.flavor.Flavor.fetch_extra_specs'
+        ) as mocked:
+            res = flavor.Flavor()
+            mocked.return_value = res
+            self._verify2(
+                'openstack.proxy.Proxy._get',
+                self.proxy.get_flavor,
+                method_args=['res', True],
+                expected_result=res,
+                expected_args=[flavor.Flavor, 'res']
+            )
+            mocked.assert_called_once()
+
+    def test_flavor_get_skip_fetch_extra(self):
+        """fetch extra_specs not triggered"""
+        with mock.patch(
+            'openstack.compute.v2.flavor.Flavor.fetch_extra_specs'
+        ) as mocked:
+            res = flavor.Flavor(extra_specs={'a': 'b'})
+            mocked.return_value = res
+            self._verify2(
+                'openstack.proxy.Proxy._get',
+                self.proxy.get_flavor,
+                method_args=['res', True],
+                expected_result=res,
+                expected_args=[flavor.Flavor, 'res']
+            )
+            mocked.assert_not_called()
 
     def test_flavors_detailed(self):
         self.verify_list(self.proxy.flavors, flavor.FlavorDetail,
                          method_kwargs={"details": True, "query": 1},
-                         expected_kwargs={"query": 1})
+                         expected_kwargs={"query": 1,
+                                          "base_path": "/flavors/detail"})
 
     def test_flavors_not_detailed(self):
         self.verify_list(self.proxy.flavors, flavor.Flavor,
                          method_kwargs={"details": False, "query": 1},
-                         expected_kwargs={"query": 1})
+                         expected_kwargs={"query": 1,
+                                          "base_path": "/flavors"})
+
+    def test_flavor_get_access(self):
+        self._verify("openstack.compute.v2.flavor.Flavor.get_access",
+                     self.proxy.get_flavor_access,
+                     method_args=["value"],
+                     expected_args=[])
+
+    def test_flavor_add_tenant_access(self):
+        self._verify("openstack.compute.v2.flavor.Flavor.add_tenant_access",
+                     self.proxy.flavor_add_tenant_access,
+                     method_args=["value", "fake-tenant"],
+                     expected_args=["fake-tenant"])
+
+    def test_flavor_remove_tenant_access(self):
+        self._verify("openstack.compute.v2.flavor.Flavor.remove_tenant_access",
+                     self.proxy.flavor_remove_tenant_access,
+                     method_args=["value", "fake-tenant"],
+                     expected_args=["fake-tenant"])
+
+    def test_flavor_fetch_extra_specs(self):
+        self._verify("openstack.compute.v2.flavor.Flavor.fetch_extra_specs",
+                     self.proxy.fetch_flavor_extra_specs,
+                     method_args=["value"],
+                     expected_args=[])
+
+    def test_create_flavor_extra_specs(self):
+        specs = {
+            'a': 'b'
+        }
+        self._verify("openstack.compute.v2.flavor.Flavor.create_extra_specs",
+                     self.proxy.create_flavor_extra_specs,
+                     method_args=["value", specs],
+                     expected_kwargs={"specs": specs})
+
+    def test_get_flavor_extra_specs_prop(self):
+        self._verify(
+            "openstack.compute.v2.flavor.Flavor.get_extra_specs_property",
+            self.proxy.get_flavor_extra_specs_property,
+            method_args=["value", "prop"],
+            expected_args=["prop"])
+
+    def test_update_flavor_extra_specs_prop(self):
+        self._verify(
+            "openstack.compute.v2.flavor.Flavor.update_extra_specs_property",
+            self.proxy.update_flavor_extra_specs_property,
+            method_args=["value", "prop", "val"],
+            expected_args=["prop", "val"])
+
+    def test_delete_flavor_extra_specs_prop(self):
+        self._verify(
+            "openstack.compute.v2.flavor.Flavor.delete_extra_specs_property",
+            self.proxy.delete_flavor_extra_specs_property,
+            method_args=["value", "prop"],
+            expected_args=["prop"])
+
+
+class TestCompute(TestComputeProxy):
+    def test_extension_find(self):
+        self.verify_find(self.proxy.find_extension, extension.Extension)
+
+    def test_extensions(self):
+        self.verify_list_no_kwargs(self.proxy.extensions, extension.Extension)
 
     def test_image_delete(self):
         self.verify_delete(self.proxy.delete_image, image.Image, False)
@@ -214,10 +355,11 @@ class TestComputeProxy(test_proxy_base.TestProxyBase):
         self.verify_get(self.proxy.get_server, server.Server)
 
     def test_servers_detailed(self):
-        self.verify_list(self.proxy.servers, server.ServerDetail,
+        self.verify_list(self.proxy.servers, server.Server,
                          method_kwargs={"details": True,
                                         "changes_since": 1, "image": 2},
-                         expected_kwargs={"changes_since": 1, "image": 2})
+                         expected_kwargs={"changes_since": 1, "image": 2,
+                                          "base_path": "/servers/detail"})
 
     def test_servers_not_detailed(self):
         self.verify_list(self.proxy.servers, server.Server,
@@ -447,6 +589,33 @@ class TestComputeProxy(test_proxy_base.TestProxyBase):
                       method_args=["value", "key"],
                       expected_args=[self.proxy, "key"])
 
+    def test_create_image(self):
+        metadata = {'k1': 'v1'}
+        with mock.patch('openstack.compute.v2.server.Server.create_image') \
+                as ci_mock:
+
+            ci_mock.return_value = 'image_id'
+            connection_mock = mock.Mock()
+            connection_mock.get_image = mock.Mock(return_value='image')
+            connection_mock.wait_for_image = mock.Mock()
+            self.proxy._connection = connection_mock
+
+            rsp = self.proxy.create_server_image(
+                'server', 'image_name', metadata, wait=True, timeout=1)
+
+            ci_mock.assert_called_with(
+                self.proxy,
+                'image_name',
+                metadata
+            )
+
+            self.proxy._connection.get_image.assert_called_with('image_id')
+            self.proxy._connection.wait_for_image.assert_called_with(
+                'image',
+                timeout=1)
+
+            self.assertEqual(connection_mock.wait_for_image(), rsp)
+
     def test_server_group_create(self):
         self.verify_create(self.proxy.create_server_group,
                            server_group.ServerGroup)
@@ -514,3 +683,81 @@ class TestComputeProxy(test_proxy_base.TestProxyBase):
                      method_args=["value", "host1", False],
                      expected_args=["host1"],
                      expected_kwargs={'force': False, 'block_migration': None})
+
+    def test_fetch_security_groups(self):
+        self._verify(
+            'openstack.compute.v2.server.Server.fetch_security_groups',
+            self.proxy.fetch_server_security_groups,
+            method_args=["value"],
+            expected_args=[])
+
+    def test_add_security_groups(self):
+        self._verify(
+            'openstack.compute.v2.server.Server.add_security_group',
+            self.proxy.add_security_group_to_server,
+            method_args=["value", {'id': 'id', 'name': 'sg'}],
+            expected_args=['sg'])
+
+    def test_remove_security_groups(self):
+        self._verify(
+            'openstack.compute.v2.server.Server.remove_security_group',
+            self.proxy.remove_security_group_from_server,
+            method_args=["value", {'id': 'id', 'name': 'sg'}],
+            expected_args=['sg'])
+
+    def test_create_server_remote_console(self):
+        self.verify_create(
+            self.proxy.create_server_remote_console,
+            server_remote_console.ServerRemoteConsole,
+            method_kwargs={"server": "test_id", "type": "fake"},
+            expected_kwargs={"server_id": "test_id", "type": "fake"})
+
+    def test_get_console_url(self):
+        self._verify(
+            'openstack.compute.v2.server.Server.get_console_url',
+            self.proxy.get_server_console_url,
+            method_args=["value", "console_type"],
+            expected_args=["console_type"])
+
+    @mock.patch('openstack.utils.supports_microversion', autospec=True)
+    @mock.patch('openstack.compute.v2._proxy.Proxy._create', autospec=True)
+    @mock.patch('openstack.compute.v2.server.Server.get_console_url',
+                autospec=True)
+    def test_create_console_mv_old(self, sgc, rcc, smv):
+        console_fake = {
+            'url': 'a',
+            'type': 'b',
+            'protocol': 'c'
+        }
+        smv.return_value = False
+        sgc.return_value = console_fake
+        ret = self.proxy.create_console('fake_server', 'fake_type')
+        smv.assert_called_once_with(self.proxy, '2.6')
+        rcc.assert_not_called()
+        sgc.assert_called_with(mock.ANY, self.proxy, 'fake_type')
+        self.assertDictEqual(console_fake, ret)
+
+    @mock.patch('openstack.utils.supports_microversion', autospec=True)
+    @mock.patch('openstack.compute.v2._proxy.Proxy._create', autospec=True)
+    @mock.patch('openstack.compute.v2.server.Server.get_console_url',
+                autospec=True)
+    def test_create_console_mv_2_6(self, sgc, rcc, smv):
+        console_fake = {
+            'url': 'a',
+            'type': 'b',
+            'protocol': 'c'
+        }
+
+        # Test server_remote_console is triggered when mv>=2.6
+        smv.return_value = True
+        rcc.return_value = server_remote_console.ServerRemoteConsole(
+            **console_fake)
+        ret = self.proxy.create_console('fake_server', 'fake_type')
+        smv.assert_called_once_with(self.proxy, '2.6')
+        sgc.assert_not_called()
+        rcc.assert_called_with(mock.ANY,
+                               server_remote_console.ServerRemoteConsole,
+                               server_id='fake_server',
+                               type='fake_type',
+                               protocol=None)
+        self.assertEqual(console_fake['url'], ret['url'])
