@@ -38,6 +38,7 @@ EXAMPLE = {
     'mac_address': '16',
     'name': '17',
     'network_id': '18',
+    'numa_affinity_policy': False,
     'port_security_enabled': True,
     'qos_network_policy_id': '32',
     'qos_policy_id': '21',
@@ -84,6 +85,7 @@ class TestPort(base.TestCase):
                               "description": "description",
                               "device_id": "device_id",
                               "device_owner": "device_owner",
+                              "fields": "fields",
                               "fixed_ips": "fixed_ips",
                               "id": "id",
                               "ip_address": "ip_address",
@@ -130,6 +132,8 @@ class TestPort(base.TestCase):
         self.assertEqual(EXAMPLE['mac_address'], sot.mac_address)
         self.assertEqual(EXAMPLE['name'], sot.name)
         self.assertEqual(EXAMPLE['network_id'], sot.network_id)
+        self.assertEqual(EXAMPLE['numa_affinity_policy'],
+                         sot.numa_affinity_policy)
         self.assertTrue(sot.is_port_security_enabled)
         self.assertEqual(EXAMPLE['qos_network_policy_id'],
                          sot.qos_network_policy_id)
